/*==============================================================================*/
/*  Copyright (C) 2009-2016, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Ethernet BitCoverage Driver                                      */
/*  EventID:   Insert EventID Test Point                                        */
/*  AppVer :   DT10 Ver10.XX`                                                  */
/*  FileID :   36C1                                                             */
/*  Version:   1.2                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for Ethernet Control                                         */
/*==============================================================================*/
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_ETHER_XXX                                                        */
/*  Desc:   Please select Ethernet mode                                         */
/*==============================================================================*/
#define DT_ETHER_TCPIP_SERVER    1
#define DT_ETHER_TCPIP_CLIENT    0
#define DT_ETHER_UDP             0

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define DT_INLINE inline
/* #define DT_INLINE static */

/*==============================================================================*/
/*  Macro:                                                                      */
/*  Desc:   Define macro for bit coverage.                                      */
/*==============================================================================*/
#define DT_UINT_BIT_SIZE        32         /* Bit width that defined the type of "DT_UINT" */
#define MAX_FILE_NUM            4096        /* Max number of addr without lower 4bit */
#define MAX_TP_NUM_PER_FILE     1024        /* Max step number in a file */
#define MAKE_KEY(addr, dat)        (((addr>>4)*MAX_TP_NUM_PER_FILE)+(dat))
#define MAX_LOG_BIT_SIZE        (((MAX_FILE_NUM * MAX_TP_NUM_PER_FILE)+(DT_UINT_BIT_SIZE-1))/ DT_UINT_BIT_SIZE)
static unsigned int logBitBuf[MAX_LOG_BIT_SIZE];


static int sock = -1;
static struct sockaddr_in ipaddr;
#if DT_ETHER_TCPIP_SERVER
static int ssock = -1;
static struct sockaddr_in sipaddr;
#endif

/*==============================================================================*/
/*  Func:   _TP_BusEthernetInit                                                 */
/*  Desc:   Please describe the code to initializes socket.                     */
/*==============================================================================*/
static void _TP_BusEthernetInit(void)
{
#if DT_ETHER_TCPIP_CLIENT
    ipaddr.sin_family = AF_INET;
    ipaddr.sin_addr.s_addr = inet_addr("192.168.10.200");
    ipaddr.sin_port = htons(5000);
#elif DT_ETHER_TCPIP_SERVER
    int n;

    sipaddr.sin_family = AF_INET;
    sipaddr.sin_addr.s_addr = INADDR_ANY;
    sipaddr.sin_port = htons(3000);
#else
    ipaddr.sin_family = AF_INET;
    ipaddr.sin_addr.s_addr = inet_addr("192.168.10.200");
    ipaddr.sin_port = htons(3000);
#endif
#if DT_ETHER_TCPIP_CLIENT
    sock = socket( AF_INET, SOCK_STREAM, 0);
#elif DT_ETHER_TCPIP_SERVER
    ssock = socket( AF_INET, SOCK_STREAM, 0);
#else
    sock = socket( AF_INET,  SOCK_DGRAM, 0 );
#endif
#if DT_ETHER_TCPIP_CLIENT
    while( 0 != connect( sock, (struct sockaddr *)&ipaddr, sizeof(ipaddr)));
#elif DT_ETHER_TCPIP_SERVER
    bind( ssock, (struct sockaddr *)&sipaddr, sizeof(sipaddr) );
    listen( ssock, 1 );
    n = sizeof( ipaddr );
    sock = accept( ssock, (struct sockaddr *)&ipaddr, &n );
#endif
}

/*==============================================================================*/
/*  Func: _TP_BusOutDrv                                                         */
/*  Desc: Test Point Output Function                                            */
/*==============================================================================*/
static int init = 0;
#if DT_ETHER_TCPIP_SERVER || DT_ETHER_TCPIP_CLIENT
DT_INLINE void _TP_BusOutDrv( unsigned char *buff, DT_UINT count )
{
    send( sock, (const char *)buff, count, 0 );
}
#else
DT_INLINE void _TP_BusOutDrv( unsigned char *buff, DT_UINT count )
{
    sendto( sock, (const char *)buff, count, 0, (struct sockaddr *)&ipaddr, sizeof(ipaddr) );
}
#endif

/*==============================================================================*/
/*     Don't change the code from here as possible.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Macro:  Command Macros                                                      */
/*  Desc:   Please do not change the value.                                     */
/*==============================================================================*/
#define DT_NRMLTP_ID        0x01        /* Normal TP */

#define DT_HEADER_SIZE        4
#define    DT_NRMLTP_SIZE        (DT_HEADER_SIZE+6)

/*==============================================================================*/
/*  Func: _TP_BusOut                                                            */
/*  Desc: Called by Test Point                                                  */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
    unsigned int key;
    key = MAKE_KEY(addr, dat);
    logBitBuf[key/DT_UINT_BIT_SIZE] |= 1 << (key & (DT_UINT_BIT_SIZE - 1));
}

/*==============================================================================*/
/*  Func: _TP_MemoryOutput                                                      */
/*  Desc: Called by Variable Test Point                                         */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
    /* Don't use this function! */
}

/*==============================================================================*/
/*  Func: _TP_DataOut                                                           */
/*  Desc: Output Test Point from buffer.                                        */
/*==============================================================================*/
void _TP_DataOut(void)
{
    unsigned int i, j;
    unsigned int addr, dat, key;
    unsigned char buff[DT_NRMLTP_SIZE];

    if( init == 0 ){
        _TP_BusEthernetInit();
        init = 1;
    }
    for( i = 0; i < MAX_LOG_BIT_SIZE; i++ ){
        for( j = 0; j < DT_UINT_BIT_SIZE; j++ ){
            if( logBitBuf[i] & (1 << j) ){
                key = (i * DT_UINT_BIT_SIZE) + j;
                dat = key % MAX_TP_NUM_PER_FILE;
                addr = ((key / MAX_TP_NUM_PER_FILE) << 4);

                buff[0] = 0xFF;                        // Header
                buff[1] = 0xFF;                        // Header
                buff[2] = DT_NRMLTP_SIZE;              // Data Size
                buff[3] = 0x00;                        // Data Size
                buff[4] = DT_NRMLTP_ID;                // Var TP
                buff[5] = dat;                         // dat
                buff[6] = dat >> 8;                    // dat
                buff[7] = addr;                        // addr
                buff[8] = addr >> 8;                   // addr
                buff[9] = addr >> 16;                  // addr
                _TP_BusOutDrv( buff, 10 );
            }
        }
    }
}

/*==============================================================================*/
